/*
 * sampleAgent.cpp
 *
 *  Created on: Aug 23, 2011
 *      Author: winkler
 *      Modified: vojta
 */


/** Example of an agent. There are two ways how the controllers are called:
  * 1. controller in separate thread, the means the a loop is executed independently on the simulator. 
  *    this loop is in task() function, which is called if start() is called in constructor. start() turns on the thread.
  *    this can be time consuming on some systems. some sleep functions should be used in task()
  * 2. controller is called in every tick by the simulator. Thus the controller does not run, if the simulator do other thing.
  *    this should be computationally less intensive and also safer.
  *    in this case, the onTickMessage() function is called every simulation tick.
  */
#include <sampleAgent.h>
#include <iostream>
#include <math.h>
#define MAX(X,Y) (X>Y?X:Y)

namespace sampleAgent {




SampleAgent::SampleAgent(srCore::Agent& agent, std::string properties) :
		HAL(agent),
		period(3000),
		precision(100),
		upperLimit(1.4),
		lowerLimit(-1.4)
{
    firstRun = 0;
 	phaseShift = atof(properties.c_str());
    std::cerr << "Sample agent constructor";

}

/** this can be used to control main hinge of the robot - usefull in organism mode */
void SampleAgent::sinusMove() {

    double time = getTime();
	float phase = (int(( (float)(time)/period + phaseShift) * precision) % MAX(1,int(precision))) / precision;
	float range = fabs(upperLimit) + fabs(lowerLimit);
	float center = (upperLimit + lowerLimit) / 2;
	moveActuator(0, sin(2*M_PI*phase) * range/2 + center);

}

/** this just moves forward/bacward the robot - usefull in swarm mode */
void SampleAgent::swarmMove() {

    double t = getTime();

    t = ((int)lround(t))% 4000;

    std::cerr << "Time = " << t << "\n";

    if (t < 1000) {
        moveForward(30);
    } else if (t < 2000) {
        moveRight(30);
    } else if (t < 3000) {
        moveForward(-30);
    } else {
        moveRight(-30);
    }

}



/** This function is called by the Simulator every simulation tick. If you want to control the robot not in thread, put your
 * stuff here 
 */
void SampleAgent::onTickMessage() {

    // we create sensors, but only for the first time
    if (firstRun == 0) {
        std::cerr << "Turning on IR sensors\n";
	    for (unsigned int i=0; i<getNumIRSensors();i++) {
    		activateIRSensor(i, true);
	    }
        firstRun = 1;
    } else {
        // we read the sensors
	    for (unsigned int i=0; i<getNumIRSensors();i++) { 
           unsigned long val;
           getIRSensorValue(i); 
        }
    }

    sinusMove();
//    swarmMove();

}


/** this function is called by thread if 'start()' was called before.
  * If you want to control the robot from threaded agent, put your stuff in this function and call start() in constructor.
  */
void SampleAgent::task() {

}

SampleAgent::~SampleAgent() {
	exit();
}

}
